from scripts import config
from scripts import version

def generate_pkg_config_file():
        f = open('./build/xmlbird.pc', 'w+')
        f.write("prefix=" + config.PREFIX + "\n")
        f.write("""exec_prefix=${prefix}
includedir=${prefix}/include
libdir=${exec_prefix}""" + config.LIBDIR + """

Name: xmlbird
Description: XML parser
Version: """ + version.XMLBIRD_VERSION + """
Cflags: -I${includedir}
Libs: -L${libdir} -lxmlbird
Requires: glib-2.0
""")



#!/usr/bin/python3

def write_compile_parameters (prefix, libdir, dest, cc, valac, non_null,
                              valacflags, cflags, ldflags):
    f = open('./scripts/config.py', 'w+')
    f.write("#!/usr/bin/python3\n")
    f.write("PREFIX =  \"" + prefix + "\"\n")
    f.write("LIBDIR =  \"" + libdir + "\"\n")
    f.write("DEST = \"" + dest + "\"\n")
    f.write("CC = \"" + cc + "\"\n")
    f.write("VALAC = \"" + valac + "\"\n")

    if non_null:
        f.write("NON_NULL = \"--enable-experimental-non-null\"\n")
    else:
        f.write("NON_NULL = \"\"\n")
        
    f.write("VALACFLAGS = " + str(valacflags) + "\n")
    f.write("CFLAGS = " + str(cflags) + "\n")
    f.write("LDFLAGS = " + str(ldflags) + "\n")



#!/usr/bin/python3
	
import subprocess
from os import path

def fuzz():
    process = subprocess.Popen ("mkdir -p build/tests", shell=True)
    process.communicate()[0]
    if not process.returncode == 0:
        print('Can not create test directory.')
        exit(1)

    fuzzed_file = 'build/tests/fuzzy.xml'
    process = subprocess.Popen ("radamsa tests/test.xml > " + fuzzed_file, shell=True)
    process.communicate()[0]
    if not process.returncode == 0:
        print('Can not create a fuzzed file. Radamsa is required.')
        exit(1)

    process = subprocess.Popen ('./run_fuzz.sh ' + fuzzed_file, shell=True)
    process.communicate()[0]
    if not process.returncode == 0:
        print("A bug was found.")
        
        i = 0
        bug_file = ''
        while path.isfile (bug_file) or bug_file == '':
            i = i + 1
            bug_file = 'build/fuzzy_bug_' + str(i) + '.xml'

        process = subprocess.Popen ('mv build/tests/fuzzy.xml ' + bug_file, shell=True)
        process.communicate()[0]
        if not process.returncode == 0:
            print('Can not move ' + bug_file)
            exit(1)
        else:
            print('Saving bug in ' + bug_file)

i = 0
while True:
    print('Running fuzz test ' + str(i))
    fuzz ()

        	
    i = i + 1



#!/usr/bin/python
"""
Copyright (C) 2013 2014 2015 Johan Mattsson

This library is free software; you can redistribute it and/or modify 
it under the terms of the GNU Lesser General Public License as 
published by the Free Software Foundation; either version 3 of the 
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
Lesser General Public License for more details.
"""

XMLBIRD_VERSION = '1.2.9'
LIBXMLBIRD_SO_VERSION_MAJOR = '1'
LIBXMLBIRD_SO_VERSION_MINOR = '0'
LIBXMLBIRD_SO_VERSION = LIBXMLBIRD_SO_VERSION_MAJOR;



import os
import glob
import subprocess
import sys

from scripts import version
from scripts import config
from scripts import tests
from scripts import pkgconfig
from scripts.builder import Builder

DOIT_CONFIG = {
    'default_tasks': [
        'libxmlbird',
        'libxmlbird_pkgconfig'
        ],
    }

all_tests = tests.all_tests ();

def soname(target_binary):
    if "darwin" in sys.platform or "msys" in sys.platform:
        return ''
        
    return '-Wl,-soname,' + target_binary
    
def make_libxmlbird(target_binary):
    valac_command = config.VALAC + """\
        -C \
        --pkg posix \
        --vapidir=./ \
        --basedir build/libxmlbird/ \
        """ + config.NON_NULL + """ \
        """ + config.VALACFLAGS.get("xmlbird", "") + """ \
        --enable-experimental \
        --library libxmlbird \
        --vapi=./build/xmlbird.vapi \
        -H build/xmlbird/xmlbird.h \
        libxmlbird/*.vala \
        """

    cc_command = config.CC + " " + config.CFLAGS.get("xmlbird", "") + """ \
            -c C_SOURCE \
            -I ./build/xmlbird \
            -fPIC \
            $(pkg-config --cflags glib-2.0) \
            -o OBJECT_FILE"""

    linker_command = config.CC + " " + config.LDFLAGS.get("xmlbird", "") + """ \
            -shared \
            """ + soname(target_binary) + """ \
            build/libxmlbird/*.o \
            $(pkg-config --libs glib-2.0) \
            $(pkg-config --libs gobject-2.0) \
            -o ./build/bin/""" + target_binary

    libxmlbird = Builder('libxmlbird',
                          valac_command, 
                          cc_command,
                          linker_command,
                          target_binary,
                          'libxmlbird.so')
			
    yield libxmlbird.build()

def task_libxmlbird():
    if "kfreebsd" in sys.platform:
        LIBXMLBIRD_SO_VERSION=version.LIBXMLBIRD_SO_VERSION
    elif "openbsd" in sys.platform:
        LIBXMLBIRD_SO_VERSION='${LIBxmlbird_VERSION}'
    else:
        LIBXMLBIRD_SO_VERSION=version.LIBXMLBIRD_SO_VERSION
    
    yield make_libxmlbird('libxmlbird.so.' + LIBXMLBIRD_SO_VERSION)

def make_libxmlbird_pkgconfig():
    pkgconfig.generate_pkg_config_file ()

def task_libxmlbird_pkgconfig():
    """build tests"""
    return {
	     'actions': [make_libxmlbird_pkgconfig],
	     'task_dep': ['libxmlbird'],
    }
    
def task_distclean ():
    return  {
        'actions': ['rm -rf .doit.db build scripts/config.py scripts/*.pyc *.pyc'],
        }

def task_build_tests():
    """build tests"""
    return {
	     'actions': [tests.build_tests],
	     'task_dep': ['libxmlbird'],
    }
    
def task_fuzz():
    """run fuzz test"""
    return {
	     'actions': ['./tests/fuzz.py'],
	     'task_dep': ['build_tests'],
    }
        
def task_test():
    """run tests"""
    
    def run_tests():
        print('Running tests')
        failed = 0
        passed = 0
        for t in all_tests:
            process = subprocess.Popen ("./run_test.sh ./build/bin/" + t, shell=True)
            process.communicate()[0]
            if not process.returncode == 0:
                    print(t + ' Failed')
                    failed = failed + 1
            else:
                    passed = passed + 1
			       
        print(str(passed) + ' tests passed and ' + str(failed) + ' failed.')
		   
    return {
	     'actions': [run_tests],
	     'task_dep': ['build_tests'],
	     'verbosity': 2
    }



from run import run

run ("mkdir -p build");

run ("valac --ccode --pkg posix --library libxmlbird --vapi=xmlbird.vapi --directory=./build -H ./build/xmlbird.h libxmlbird/*.vala");

run ("""gcc -fPIC -c \
	$(pkg-config --cflags glib-2.0) \
	$(pkg-config --cflags gobject-2.0) \
	build/libxmlbird/*.c""");

run ("mv *.o build/libxmlbird/");

run ("""gcc -shared \
		build/libxmlbird/*.o \
		$(pkg-config --libs glib-2.0) \
		$(pkg-config --libs gobject-2.0) \
		-o build/libxmlbird.dll""");
		


#!/usr/bin/python
import subprocess
from scripts import tests

tests = tests.get_tests();

for test in tests:
    print('Running tests:')
    process = subprocess.Popen("./run_test.sh ./build/bin/" + test, shell=True)
    process.communicate()[0]
    
    if not process.returncode == 0:
        print(test + ' failed')
        exit (1)
    else:
        print(test + ' passed')
    
